# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

from dataclasses import asdict, dataclass
from typing import Literal

from ..auth import Auth, Config
from ..listable import ListableResource


@dataclass
class Webhook:
    """

    Attributes
    ----------
    createdAt: str
      The UTC timestamp when the webhook was created
    createdByUserId: str
      The id of the user that created the webhook
    description: str
      The reason for creating the webhook
    disabledReason: str
      The reason for the webhook to be disabled
    disabledReasonCode: str
      The unique code for the reason
    enabled: bool
      Whether the webhook is active and sending requests
    eventTypes: list[str]
      Types of events for which the webhook should trigger.
    filter: str
      Filter that should match for a webhook to be triggered.
      Supported common attribute names are 'id', 'spaceId' and 'topLevelResourceId', beside the common attributes the "com.qlik.v1.app.reload.finished" event also supports "data.status" that could be either "ok" or "error" but can't be used together with other event types.
      Supported attribute operators are 'eq' and 'ne'.
      Supported logical operators are 'and' and 'or'.
      Note that attribute values must be valid JSON strings, hence they're enclosed with double quotes
      For more detailed information regarding the SCIM filter syntax (RFC7644) used please follow the link to external documentation.
    headers: object
      Additional headers in the post request
    id: str
      The webhook's unique identifier
    level: Literal["tenant", "user"]
      Defines at what level the webhook should operate: for all resources belonging to a tenant or restricted to only those accessible by the webhook-creator.
    name: str
      The name for the webhook
    ownerId: str
      The id of the user that owns the webhook, only applicable for user level webhooks
    secret: str
      String used as secret for calculating HMAC hash sent as header
    updatedAt: str
      The UTC timestamp when the webhook was last updated
    updatedByUserId: str
      The id of the user that last updated the webhook
    url: str
      Target URL for webhook HTTPS requests
    """

    createdAt: str = None
    createdByUserId: str = None
    description: str = None
    disabledReason: str = None
    disabledReasonCode: str = None
    enabled: bool = None
    eventTypes: list[str] = None
    filter: str = None
    headers: object = None
    id: str = None
    level: Literal["tenant", "user"] = "tenant"
    name: str = None
    ownerId: str = None
    secret: str = None
    updatedAt: str = None
    updatedByUserId: str = None
    url: str = None

    def __init__(self_, **kvargs):
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "createdByUserId" in kvargs and kvargs["createdByUserId"] is not None:
            self_.createdByUserId = kvargs["createdByUserId"]
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "disabledReason" in kvargs and kvargs["disabledReason"] is not None:
            self_.disabledReason = kvargs["disabledReason"]
        if "disabledReasonCode" in kvargs and kvargs["disabledReasonCode"] is not None:
            self_.disabledReasonCode = kvargs["disabledReasonCode"]
        if "enabled" in kvargs and kvargs["enabled"] is not None:
            self_.enabled = kvargs["enabled"]
        if "eventTypes" in kvargs and kvargs["eventTypes"] is not None:
            self_.eventTypes = kvargs["eventTypes"]
        if "filter" in kvargs and kvargs["filter"] is not None:
            self_.filter = kvargs["filter"]
        if "headers" in kvargs and kvargs["headers"] is not None:
            self_.headers = kvargs["headers"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "level" in kvargs and kvargs["level"] is not None:
            self_.level = kvargs["level"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "ownerId" in kvargs and kvargs["ownerId"] is not None:
            self_.ownerId = kvargs["ownerId"]
        if "secret" in kvargs and kvargs["secret"] is not None:
            self_.secret = kvargs["secret"]
        if "updatedAt" in kvargs and kvargs["updatedAt"] is not None:
            self_.updatedAt = kvargs["updatedAt"]
        if "updatedByUserId" in kvargs and kvargs["updatedByUserId"] is not None:
            self_.updatedByUserId = kvargs["updatedByUserId"]
        if "url" in kvargs and kvargs["url"] is not None:
            self_.url = kvargs["url"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def resend_delivery(self, deliveryId: str) -> Delivery:
        """
        Resend the delivery with the same payload

        Parameters
        ----------
        deliveryId: str
          The delivery's unique identifier.
        """
        response = self.auth.rest(
            path="/webhooks/{id}/deliveries/{deliveryId}/actions/resend".replace(
                "{deliveryId}", deliveryId
            ).replace("{id}", self.id),
            method="POST",
            params={},
            data=None,
        )
        obj = Delivery(**response.json())
        obj.auth = self.auth
        return obj

    def get_delivery(self, deliveryId: str) -> Delivery:
        """
        Returns details for a specific delivery

        Parameters
        ----------
        deliveryId: str
          The delivery's unique identifier.
        """
        response = self.auth.rest(
            path="/webhooks/{id}/deliveries/{deliveryId}".replace(
                "{deliveryId}", deliveryId
            ).replace("{id}", self.id),
            method="GET",
            params={},
            data=None,
        )
        obj = Delivery(**response.json())
        obj.auth = self.auth
        return obj

    def get_deliveries(
        self,
        eventType: str = None,
        limit: float = 20,
        next: str = None,
        prev: str = None,
        sort: Literal[
            "status",
            "+status",
            "-status",
            "triggeredAt",
            "+triggeredAt",
            "-triggeredAt",
        ] = "-triggeredAt",
        status: Literal["success", "fail"] = None,
    ) -> ListableResource[Delivery]:
        """
        Returns deliveries for a specific webhook

        Parameters
        ----------
        eventType: str = None
          Filter resources by event-type
        limit: float = 20
          Maximum number of deliveries to retrieve
        next: str = None
          Cursor to the next page
        prev: str = None
          Cursor to previous next page
        sort: Literal["status", "+status", "-status", "triggeredAt", "+triggeredAt", "-triggeredAt"] = "-triggeredAt"
          Field to sort by, prefix with -/+ to indicate order
        status: Literal["success", "fail"] = None
          Filter resources by status (success or fail)
        """
        query_params = {}
        if eventType is not None:
            query_params["eventType"] = eventType
        if limit is not None:
            query_params["limit"] = limit
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev
        if sort is not None:
            query_params["sort"] = sort
        if status is not None:
            query_params["status"] = status
        response = self.auth.rest(
            path="/webhooks/{id}/deliveries".replace("{id}", self.id),
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=Delivery,
            auth=self.auth,
            path="/webhooks/{id}/deliveries".replace("{id}", self.id),
            query_params=query_params,
        )

    def delete(self) -> None:
        """
        Deletes a specific webhook

        Parameters
        ----------
        """
        self.auth.rest(
            path="/webhooks/{id}".replace("{id}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def patch(self, WebhookPatchData: list[WebhookPatch]) -> None:
        """
        Patches a webhook

        Parameters
        ----------
        WebhookPatchData: list[WebhookPatch]
        """
        data = [asdict(elem) for elem in WebhookPatchData]
        self.auth.rest(
            path="/webhooks/{id}".replace("{id}", self.id),
            method="PATCH",
            params={},
            data=data,
        )

    def set(self, data: WebhookDef) -> Webhook:
        """
        Updates a webhook

        Parameters
        ----------
        data: WebhookDef
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/webhooks/{id}".replace("{id}", self.id),
            method="PUT",
            params={},
            data=data,
        )
        self.__init__(**response.json())
        return self


@dataclass
class Delivery:
    """

    Attributes
    ----------
    eventType: str
      The name of the triggering event-type
    id: str
      The delivery's unique identifier
    request: DeliveryRequest
    response: DeliveryResponse
    status: Literal["success", "fail"]
      The status of delivery
    statusMessage: str
      The status message of the delivery
    triggeredAt: str
      The UTC timestamp when the delivery was triggered
    webhookId: str
      The unique webhook identifier that the delivery is for
    """

    eventType: str = None
    id: str = None
    request: DeliveryRequest = None
    response: DeliveryResponse = None
    status: Literal["success", "fail"] = None
    statusMessage: str = None
    triggeredAt: str = None
    webhookId: str = None

    def __init__(self_, **kvargs):
        if "eventType" in kvargs and kvargs["eventType"] is not None:
            self_.eventType = kvargs["eventType"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "request" in kvargs and kvargs["request"] is not None:
            if type(kvargs["request"]).__name__ == Delivery.__annotations__["request"]:
                self_.request = kvargs["request"]
            else:
                self_.request = DeliveryRequest(**kvargs["request"])
        if "response" in kvargs and kvargs["response"] is not None:
            if (
                type(kvargs["response"]).__name__
                == Delivery.__annotations__["response"]
            ):
                self_.response = kvargs["response"]
            else:
                self_.response = DeliveryResponse(**kvargs["response"])
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        if "statusMessage" in kvargs and kvargs["statusMessage"] is not None:
            self_.statusMessage = kvargs["statusMessage"]
        if "triggeredAt" in kvargs and kvargs["triggeredAt"] is not None:
            self_.triggeredAt = kvargs["triggeredAt"]
        if "webhookId" in kvargs and kvargs["webhookId"] is not None:
            self_.webhookId = kvargs["webhookId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DeliveryList:
    """

    Attributes
    ----------
    data: list[Delivery]
    links: DeliveryListLinks
    """

    data: list[Delivery] = None
    links: DeliveryListLinks = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == DeliveryList.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [Delivery(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == DeliveryList.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = DeliveryListLinks(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DeliveryListLinks:
    """

    Attributes
    ----------
    next: Link
    prev: Link
    self: Link
    """

    next: Link = None
    prev: Link = None
    self: Link = None

    def __init__(self_, **kvargs):
        if "next" in kvargs and kvargs["next"] is not None:
            if (
                type(kvargs["next"]).__name__
                == DeliveryListLinks.__annotations__["next"]
            ):
                self_.next = kvargs["next"]
            else:
                self_.next = Link(**kvargs["next"])
        if "prev" in kvargs and kvargs["prev"] is not None:
            if (
                type(kvargs["prev"]).__name__
                == DeliveryListLinks.__annotations__["prev"]
            ):
                self_.prev = kvargs["prev"]
            else:
                self_.prev = Link(**kvargs["prev"])
        if "self" in kvargs and kvargs["self"] is not None:
            if (
                type(kvargs["self"]).__name__
                == DeliveryListLinks.__annotations__["self"]
            ):
                self_.self = kvargs["self"]
            else:
                self_.self = Link(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DeliveryRequest:
    """

    Attributes
    ----------
    body: object
      The sent body/payload of the delivery
    headers: object
      Headers sent for this delivery
    url: str
      URL used for this delivery
    """

    body: object = None
    headers: object = None
    url: str = None

    def __init__(self_, **kvargs):
        if "body" in kvargs and kvargs["body"] is not None:
            self_.body = kvargs["body"]
        if "headers" in kvargs and kvargs["headers"] is not None:
            self_.headers = kvargs["headers"]
        if "url" in kvargs and kvargs["url"] is not None:
            self_.url = kvargs["url"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DeliveryResponse:
    """

    Attributes
    ----------
    body: str
      The received body of the delivery
    headers: object
      Headers received for this delivery
    statusCode: float
      The HTTP status code of the response
    """

    body: str = None
    headers: object = None
    statusCode: float = None

    def __init__(self_, **kvargs):
        if "body" in kvargs and kvargs["body"] is not None:
            self_.body = kvargs["body"]
        if "headers" in kvargs and kvargs["headers"] is not None:
            self_.headers = kvargs["headers"]
        if "statusCode" in kvargs and kvargs["statusCode"] is not None:
            self_.statusCode = kvargs["statusCode"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class EventType:
    """

    Attributes
    ----------
    description: str
      Description of the event type
    levels: list[str]
      Specifies which levels that are supported for this event type
    name: str
      Name of the event type
    title: str
      Title of the event type
    """

    description: str = None
    levels: list[str] = None
    name: str = None
    title: str = None

    def __init__(self_, **kvargs):
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "levels" in kvargs and kvargs["levels"] is not None:
            self_.levels = kvargs["levels"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "title" in kvargs and kvargs["title"] is not None:
            self_.title = kvargs["title"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class EventTypes:
    """

    Attributes
    ----------
    data: list[EventType]
    """

    data: list[EventType] = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == EventTypes.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [EventType(**e) for e in kvargs["data"]]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Link:
    """

    Attributes
    ----------
    href: str
      URL to a resource request
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class WebhookDef:
    """

    Attributes
    ----------
    description: str
      The reason for creating the webhook
    enabled: bool
      Whether the webhook is active and sending requests
    eventTypes: list[str]
      Types of events for which the webhook should trigger.
    filter: str
      Filter that should match for a webhook to be triggered.
      Supported common attribute names are 'id', 'spaceId' and 'topLevelResourceId', beside the common attributes the "com.qlik.v1.app.reload.finished" event also supports "data.status" that could be either "ok" or "error" but can't be used together with other event types.
      Supported attribute operators are 'eq' and 'ne'.
      Supported logical operators are 'and' and 'or'.
      Note that attribute values must be valid JSON strings, hence they're enclosed with double quotes
      For more detailed information regarding the SCIM filter syntax (RFC7644) used please follow the link to external documentation.
    headers: object
      Additional headers in the post request
    level: Literal["tenant", "user"]
      Defines at what level the webhook should operate: for all resources belonging to a tenant or restricted to only those accessible by the webhook-creator.
    name: str
      The name for the webhook
    ownerId: str
      The id of the user that owns the webhook, only applicable for user level webhooks
    secret: str
      String used as secret for calculating HMAC hash sent as header
    url: str
      Target URL for webhook HTTPS requests
    """

    description: str = None
    enabled: bool = None
    eventTypes: list[str] = None
    filter: str = None
    headers: object = None
    level: Literal["tenant", "user"] = "tenant"
    name: str = None
    ownerId: str = None
    secret: str = None
    url: str = None

    def __init__(self_, **kvargs):
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "enabled" in kvargs and kvargs["enabled"] is not None:
            self_.enabled = kvargs["enabled"]
        if "eventTypes" in kvargs and kvargs["eventTypes"] is not None:
            self_.eventTypes = kvargs["eventTypes"]
        if "filter" in kvargs and kvargs["filter"] is not None:
            self_.filter = kvargs["filter"]
        if "headers" in kvargs and kvargs["headers"] is not None:
            self_.headers = kvargs["headers"]
        if "level" in kvargs and kvargs["level"] is not None:
            self_.level = kvargs["level"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "ownerId" in kvargs and kvargs["ownerId"] is not None:
            self_.ownerId = kvargs["ownerId"]
        if "secret" in kvargs and kvargs["secret"] is not None:
            self_.secret = kvargs["secret"]
        if "url" in kvargs and kvargs["url"] is not None:
            self_.url = kvargs["url"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class WebhookList:
    """

    Attributes
    ----------
    data: list[Webhook]
    links: WebhookListLinks
    """

    data: list[Webhook] = None
    links: WebhookListLinks = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == WebhookList.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [Webhook(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == WebhookList.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = WebhookListLinks(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class WebhookListLinks:
    """

    Attributes
    ----------
    next: Link
    prev: Link
    self: Link
    """

    next: Link = None
    prev: Link = None
    self: Link = None

    def __init__(self_, **kvargs):
        if "next" in kvargs and kvargs["next"] is not None:
            if (
                type(kvargs["next"]).__name__
                == WebhookListLinks.__annotations__["next"]
            ):
                self_.next = kvargs["next"]
            else:
                self_.next = Link(**kvargs["next"])
        if "prev" in kvargs and kvargs["prev"] is not None:
            if (
                type(kvargs["prev"]).__name__
                == WebhookListLinks.__annotations__["prev"]
            ):
                self_.prev = kvargs["prev"]
            else:
                self_.prev = Link(**kvargs["prev"])
        if "self" in kvargs and kvargs["self"] is not None:
            if (
                type(kvargs["self"]).__name__
                == WebhookListLinks.__annotations__["self"]
            ):
                self_.self = kvargs["self"]
            else:
                self_.self = Link(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class WebhookPatch:
    """
    A JSON Patch document as defined in https://datatracker.ietf.org/doc/html/rfc6902

    Attributes
    ----------
    op: Literal["add", "remove", "replace"]
      The operation to be performed
    path: Literal["/name", "/description", "/url", "/eventTypes", "/headers", "/enabled", "/secret"]
      The path for the given resource field to patch
    value: object
      The value to be used for this operation.
    """

    op: Literal["add", "remove", "replace"] = None
    path: Literal[
        "/name",
        "/description",
        "/url",
        "/eventTypes",
        "/headers",
        "/enabled",
        "/secret",
    ] = None
    value: bool | int | object | str = None

    def __init__(self_, **kvargs):
        if "op" in kvargs and kvargs["op"] is not None:
            self_.op = kvargs["op"]
        if "path" in kvargs and kvargs["path"] is not None:
            self_.path = kvargs["path"]
        if "value" in kvargs and kvargs["value"] is not None:
            self_.value = kvargs["value"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Webhooks:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_event_types(self) -> EventTypes:
        """
        List of event-types that are possible to subscribe to.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/webhooks/event-types",
            method="GET",
            params={},
            data=None,
        )
        obj = EventTypes(**response.json())
        obj.auth = self.auth
        return obj

    def get(self, id: str) -> Webhook:
        """
        Returns details for a specific webhook

        Parameters
        ----------
        id: str
          The webhook's unique identifier.
        """
        response = self.auth.rest(
            path="/webhooks/{id}".replace("{id}", id),
            method="GET",
            params={},
            data=None,
        )
        obj = Webhook(**response.json())
        obj.auth = self.auth
        return obj

    def get_webhooks(
        self,
        createdByUserId: str = None,
        enabled: bool = None,
        eventType: str = None,
        level: str = None,
        limit: float = 20,
        name: str = None,
        next: str = None,
        ownerId: str = None,
        prev: str = None,
        sort: Literal[
            "name",
            "+name",
            "-name",
            "url",
            "+url",
            "-url",
            "createdAt",
            "+createdAt",
            "-createdAt",
            "updatedAt",
            "+updatedAt",
            "-updatedAt",
        ] = "-createdAt",
        updatedByUserId: str = None,
        url: str = None,
    ) -> ListableResource[Webhook]:
        """
        Retrieves all webhooks entries for a tenant

        Parameters
        ----------
        createdByUserId: str = None
          Filter resources by user that created it
        enabled: bool = None
          Filter resources by enabled true/false
        eventType: str = None
          Filter resources by event-type
        level: str = None
          Filter resources by level that user has access to (either user or level)
        limit: float = 20
          Maximum number of webhooks to retrieve
        name: str = None
          Filter resources by name (wildcard and case insensitive)
        next: str = None
          Cursor to the next page
        ownerId: str = None
          Filter resources by user that owns it, only applicable for user level webhooks
        prev: str = None
          Cursor to previous next page
        sort: Literal["name", "+name", "-name", "url", "+url", "-url", "createdAt", "+createdAt", "-createdAt", "updatedAt", "+updatedAt", "-updatedAt"] = "-createdAt"
          Field to sort by, prefix with -/+ to indicate order
        updatedByUserId: str = None
          Filter resources by user that last updated the webhook
        url: str = None
          Filter resources by url (wildcard and case insensitive)
        """
        query_params = {}
        if createdByUserId is not None:
            query_params["createdByUserId"] = createdByUserId
        if enabled is not None:
            query_params["enabled"] = enabled
        if eventType is not None:
            query_params["eventType"] = eventType
        if level is not None:
            query_params["level"] = level
        if limit is not None:
            query_params["limit"] = limit
        if name is not None:
            query_params["name"] = name
        if next is not None:
            query_params["next"] = next
        if ownerId is not None:
            query_params["ownerId"] = ownerId
        if prev is not None:
            query_params["prev"] = prev
        if sort is not None:
            query_params["sort"] = sort
        if updatedByUserId is not None:
            query_params["updatedByUserId"] = updatedByUserId
        if url is not None:
            query_params["url"] = url
        response = self.auth.rest(
            path="/webhooks",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=Webhook,
            auth=self.auth,
            path="/webhooks",
            query_params=query_params,
        )

    def create(self, data: WebhookDef) -> Webhook:
        """
        Creates a new webhook

        Parameters
        ----------
        data: WebhookDef
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/webhooks",
            method="POST",
            params={},
            data=data,
        )
        obj = Webhook(**response.json())
        obj.auth = self.auth
        return obj



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

import io
import json
from dataclasses import asdict, dataclass

from ..auth import Auth, Config
from ..utils import get_mime_type


@dataclass
class Extension:
    """
    The extension model.

    Attributes
    ----------
    author: str
      Author of the extension.
    bundle: BundleMeta
      Object containing meta data regarding the bundle the extension belongs to. If it does not belong to a bundle, this object is not defined.
    bundled: bool
      If the extension is part of an extension bundle.
    checksum: str
      Checksum of the extension contents.
    createdAt: str
    dependencies: object
      Map of dependencies describing version of the component it requires.
    deprecated: str
      A date noting when the extension was deprecated.
    description: str
      Description of the extension.
    file: object
      The file that was uploaded with the extension.
    homepage: str
      Home page of the extension.
    icon: str
      Icon to show in the client.
    id: str
    keywords: str
      Keywords for the extension.
    license: str
      Under which license this extension is published.
    loadpath: str
      Relative path to the extension's entry file, defaults to `filename` from the qext file.
    name: str
      The display name of this extension.
    preview: str
      Path to an image that enables users to preview the extension.
    qextFilename: str
      The name of the qext file that was uploaded with this extension.
    qextVersion: str
      The version from the qext file that was uploaded with this extension.
    repository: str
      Link to the extension source code.
    supernova: bool
      If the extension is a supernova extension or not.
    supplier: str
      Supplier of the extension.
    tags: list[str]
      List of tags.
    tenantId: str
    type: str
      The type of this extension (visualization, etc.).
    updateAt: str
    userId: str
    version: str
      Version of the extension.
    """

    author: str = None
    bundle: BundleMeta = None
    bundled: bool = None
    checksum: str = None
    createdAt: str = None
    dependencies: object = None
    deprecated: str = None
    description: str = None
    file: object = None
    homepage: str = None
    icon: str = None
    id: str = None
    keywords: str = None
    license: str = None
    loadpath: str = None
    name: str = None
    preview: str = None
    qextFilename: str = None
    qextVersion: str = None
    repository: str = None
    supernova: bool = None
    supplier: str = None
    tags: list[str] = None
    tenantId: str = None
    type: str = None
    updateAt: str = None
    userId: str = None
    version: str = None

    def __init__(self_, **kvargs):
        if "author" in kvargs and kvargs["author"] is not None:
            self_.author = kvargs["author"]
        if "bundle" in kvargs and kvargs["bundle"] is not None:
            if type(kvargs["bundle"]).__name__ == Extension.__annotations__["bundle"]:
                self_.bundle = kvargs["bundle"]
            else:
                self_.bundle = BundleMeta(**kvargs["bundle"])
        if "bundled" in kvargs and kvargs["bundled"] is not None:
            self_.bundled = kvargs["bundled"]
        if "checksum" in kvargs and kvargs["checksum"] is not None:
            self_.checksum = kvargs["checksum"]
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "dependencies" in kvargs and kvargs["dependencies"] is not None:
            self_.dependencies = kvargs["dependencies"]
        if "deprecated" in kvargs and kvargs["deprecated"] is not None:
            self_.deprecated = kvargs["deprecated"]
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "file" in kvargs and kvargs["file"] is not None:
            self_.file = kvargs["file"]
        if "homepage" in kvargs and kvargs["homepage"] is not None:
            self_.homepage = kvargs["homepage"]
        if "icon" in kvargs and kvargs["icon"] is not None:
            self_.icon = kvargs["icon"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "keywords" in kvargs and kvargs["keywords"] is not None:
            self_.keywords = kvargs["keywords"]
        if "license" in kvargs and kvargs["license"] is not None:
            self_.license = kvargs["license"]
        if "loadpath" in kvargs and kvargs["loadpath"] is not None:
            self_.loadpath = kvargs["loadpath"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "preview" in kvargs and kvargs["preview"] is not None:
            self_.preview = kvargs["preview"]
        if "qextFilename" in kvargs and kvargs["qextFilename"] is not None:
            self_.qextFilename = kvargs["qextFilename"]
        if "qextVersion" in kvargs and kvargs["qextVersion"] is not None:
            self_.qextVersion = kvargs["qextVersion"]
        if "repository" in kvargs and kvargs["repository"] is not None:
            self_.repository = kvargs["repository"]
        if "supernova" in kvargs and kvargs["supernova"] is not None:
            self_.supernova = kvargs["supernova"]
        if "supplier" in kvargs and kvargs["supplier"] is not None:
            self_.supplier = kvargs["supplier"]
        if "tags" in kvargs and kvargs["tags"] is not None:
            self_.tags = kvargs["tags"]
        if "tenantId" in kvargs and kvargs["tenantId"] is not None:
            self_.tenantId = kvargs["tenantId"]
        if "type" in kvargs and kvargs["type"] is not None:
            self_.type = kvargs["type"]
        if "updateAt" in kvargs and kvargs["updateAt"] is not None:
            self_.updateAt = kvargs["updateAt"]
        if "userId" in kvargs and kvargs["userId"] is not None:
            self_.userId = kvargs["userId"]
        if "version" in kvargs and kvargs["version"] is not None:
            self_.version = kvargs["version"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def get_file_by_filepath(self, filepath: str) -> None:
        """
        Downloads a file from the extension archive.

        Parameters
        ----------
        filepath: str
          Path to the file archive for the specified extension archive. Folders separated with forward slashes.
        """
        self.auth.rest(
            path="/extensions/{id}/file/{filepath}".replace(
                "{filepath}", filepath
            ).replace("{id}", self.id),
            method="GET",
            params={},
            data=None,
        )

    def get_file(self) -> None:
        """
        Downloads the extension as an archive.

        Parameters
        ----------
        """
        self.auth.rest(
            path="/extensions/{id}/file".replace("{id}", self.id),
            method="GET",
            params={},
            data=None,
        )

    def delete(self) -> None:
        """
        Deletes a specific extension.

        Parameters
        ----------
        """
        self.auth.rest(
            path="/extensions/{id}".replace("{id}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def patch(
        self, data: ExtensionDef = None, file: io.BufferedReader = None
    ) -> Extension:
        """
        Updates a specific extension with provided data. If a file is provided, the data field is not required.

        Parameters
        ----------
        data: ExtensionDef = None
        file: str = None
          Extension archive.
        """
        files_dict = {}
        files_dict["file"] = ("file", file, get_mime_type(file))
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
            files_dict["data"] = (None, json.dumps(data))
        response = self.auth.rest(
            path="/extensions/{id}".replace("{id}", self.id),
            method="PATCH",
            params={},
            data=None,
            files=files_dict,
        )
        self.__init__(**response.json())
        return self


@dataclass
class ExtensionDef:
    """
    The extension model.

    Attributes
    ----------
    author: str
      Author of the extension.
    bundle: BundleMeta
      Object containing meta data regarding the bundle the extension belongs to. If it does not belong to a bundle, this object is not defined.
    bundled: bool
      If the extension is part of an extension bundle.
    checksum: str
      Checksum of the extension contents.
    dependencies: object
      Map of dependencies describing version of the component it requires.
    deprecated: str
      A date noting when the extension was deprecated.
    description: str
      Description of the extension.
    file: object
      The file that was uploaded with the extension.
    homepage: str
      Home page of the extension.
    icon: str
      Icon to show in the client.
    keywords: str
      Keywords for the extension.
    license: str
      Under which license this extension is published.
    loadpath: str
      Relative path to the extension's entry file, defaults to `filename` from the qext file.
    name: str
      The display name of this extension.
    preview: str
      Path to an image that enables users to preview the extension.
    qextFilename: str
      The name of the qext file that was uploaded with this extension.
    qextVersion: str
      The version from the qext file that was uploaded with this extension.
    repository: str
      Link to the extension source code.
    supernova: bool
      If the extension is a supernova extension or not.
    supplier: str
      Supplier of the extension.
    tags: list[str]
      List of tags.
    type: str
      The type of this extension (visualization, etc.).
    version: str
      Version of the extension.
    """

    author: str = None
    bundle: BundleMeta = None
    bundled: bool = None
    checksum: str = None
    dependencies: object = None
    deprecated: str = None
    description: str = None
    file: object = None
    homepage: str = None
    icon: str = None
    keywords: str = None
    license: str = None
    loadpath: str = None
    name: str = None
    preview: str = None
    qextFilename: str = None
    qextVersion: str = None
    repository: str = None
    supernova: bool = None
    supplier: str = None
    tags: list[str] = None
    type: str = None
    version: str = None

    def __init__(self_, **kvargs):
        if "author" in kvargs and kvargs["author"] is not None:
            self_.author = kvargs["author"]
        if "bundle" in kvargs and kvargs["bundle"] is not None:
            if (
                type(kvargs["bundle"]).__name__
                == ExtensionDef.__annotations__["bundle"]
            ):
                self_.bundle = kvargs["bundle"]
            else:
                self_.bundle = BundleMeta(**kvargs["bundle"])
        if "bundled" in kvargs and kvargs["bundled"] is not None:
            self_.bundled = kvargs["bundled"]
        if "checksum" in kvargs and kvargs["checksum"] is not None:
            self_.checksum = kvargs["checksum"]
        if "dependencies" in kvargs and kvargs["dependencies"] is not None:
            self_.dependencies = kvargs["dependencies"]
        if "deprecated" in kvargs and kvargs["deprecated"] is not None:
            self_.deprecated = kvargs["deprecated"]
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "file" in kvargs and kvargs["file"] is not None:
            self_.file = kvargs["file"]
        if "homepage" in kvargs and kvargs["homepage"] is not None:
            self_.homepage = kvargs["homepage"]
        if "icon" in kvargs and kvargs["icon"] is not None:
            self_.icon = kvargs["icon"]
        if "keywords" in kvargs and kvargs["keywords"] is not None:
            self_.keywords = kvargs["keywords"]
        if "license" in kvargs and kvargs["license"] is not None:
            self_.license = kvargs["license"]
        if "loadpath" in kvargs and kvargs["loadpath"] is not None:
            self_.loadpath = kvargs["loadpath"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "preview" in kvargs and kvargs["preview"] is not None:
            self_.preview = kvargs["preview"]
        if "qextFilename" in kvargs and kvargs["qextFilename"] is not None:
            self_.qextFilename = kvargs["qextFilename"]
        if "qextVersion" in kvargs and kvargs["qextVersion"] is not None:
            self_.qextVersion = kvargs["qextVersion"]
        if "repository" in kvargs and kvargs["repository"] is not None:
            self_.repository = kvargs["repository"]
        if "supernova" in kvargs and kvargs["supernova"] is not None:
            self_.supernova = kvargs["supernova"]
        if "supplier" in kvargs and kvargs["supplier"] is not None:
            self_.supplier = kvargs["supplier"]
        if "tags" in kvargs and kvargs["tags"] is not None:
            self_.tags = kvargs["tags"]
        if "type" in kvargs and kvargs["type"] is not None:
            self_.type = kvargs["type"]
        if "version" in kvargs and kvargs["version"] is not None:
            self_.version = kvargs["version"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class BundleMeta:
    """
    Object containing meta data regarding the bundle the extension belongs to. If it does not belong to a bundle, this object is not defined.

    Attributes
    ----------
    description: str
      Description of the bundle.
    id: str
      Unique identifier of the bundle.
    name: str
      Name of the bundle.
    """

    description: str = None
    id: str = None
    name: str = None

    def __init__(self_, **kvargs):
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ExtensionsClass:
    """

    Attributes
    ----------
    data: list[Extension]
    """

    data: list[Extension] = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == ExtensionsClass.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [Extension(**e) for e in kvargs["data"]]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Extensions:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get(self, id: str) -> Extension:
        """
        Returns a specific extension.

        Parameters
        ----------
        id: str
          Extension identifier or its qextFilename.
        """
        response = self.auth.rest(
            path="/extensions/{id}".replace("{id}", id),
            method="GET",
            params={},
            data=None,
        )
        obj = Extension(**response.json())
        obj.auth = self.auth
        return obj

    def get_extensions(self) -> ExtensionsClass:
        """
        Lists all extensions.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/extensions",
            method="GET",
            params={},
            data=None,
        )
        obj = ExtensionsClass(**response.json())
        obj.auth = self.auth
        return obj

    def create(
        self, data: ExtensionDef = None, file: io.BufferedReader = None
    ) -> Extension:
        """
        Creates a new extension. If a file is provided, the data field is not required.

        Parameters
        ----------
        data: ExtensionDef = None
        file: str = None
          Extension archive.
        """
        files_dict = {}
        files_dict["file"] = ("file", file, get_mime_type(file))
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
            files_dict["data"] = (None, json.dumps(data))
        response = self.auth.rest(
            path="/extensions", method="POST", params={}, data=None, files=files_dict
        )
        obj = Extension(**response.json())
        obj.auth = self.auth
        return obj



import base64
import enum
import platform
from dataclasses import dataclass
from functools import reduce
from typing import Callable

import requests

from ._version import __version__
from .auth_type import AuthType
from .config import Config
from .interceptors.types import InterceptorHandler, Interceptors


class HTTPMethods(enum.Enum):
    "GET",
    "OPTIONS",
    "HEAD",
    "POST",
    "PUT",
    "PATCH",
    "DELETE"


class NoUrlException(Exception):
    """
    NoUrlException represents a missing URL
    """


class ConnectionException(Exception):
    """
    ConnectionException represents a failure in the request
    The request raised an exception
    ConnectionException is not a 4xx response
    """


class AuthenticationException(Exception):
    """
    AuthenticationException represents a failure to authenticate
    """


def _get_dict(o: any) -> dict:
    """
    get_dict attempts to convert objects to dict, nested
    """
    try:
        if isinstance(o, enum.Enum):
            return o.value
        if isinstance(o, dict):
            d = o
        else:
            d = o.__dict__
        d2 = {}
        for k in d:
            if d[k] is not None:
                d2[k] = _get_dict(d[k])
        return d2
    except Exception:
        return o


RequestInterceptor = Callable[[requests.Request], requests.Request]
""" REST Request interceptor """
ResponseInterceptor = Callable[[requests.Response], requests.Response]
""" REST Request interceptor """


@dataclass
class RestClient:
    """
    RestClient represents a rest client for making api calls
    """

    config: Config
    base_url: str
    _interceptors: Interceptors[ResponseInterceptor, RequestInterceptor] = None

    def __init__(self, config) -> None:
        self.config = config
        self.base_url = config.host
        self._interceptors = dict(
            request=InterceptorHandler[RequestInterceptor](),
            response=InterceptorHandler[ResponseInterceptor](),
        )

    def rest(
        self,
        path: str,
        method: HTTPMethods = "GET",
        data=None,
        files=None,
        params: dict = None,
        headers: dict = None,
        stream: bool = False,
        timeout: int = 10,
    ) -> requests.Response:
        """
        rest sends a request
        """
        if not self.base_url:
            raise NoUrlException("Caller has no 'base_url'")
        if not path.lower().startswith("http://") and not path.lower().startswith(
            "https://"
        ):
            exclude_v1_api_list = [
                "/api/v1",
                "/login/jwt-session",
                "/oauth/token",
                "/oauth/authorize",
                "/oauth/revoke",
            ]
            should_add_prefix = any(
                path.lower().startswith(value) for value in exclude_v1_api_list
            )
            path = f"{ '/api/v1' if not should_add_prefix else ''}{path}"
        else:
            path = path.split(self.base_url)[1]
        if headers is None:
            headers = {}
        # Add authorization depending upon type of auth flow
        if self.config.auth_type == AuthType.APIKey:
            headers["authorization"] = "Bearer " + self.config.api_key
        elif self.config.auth_type == AuthType.OAuth2:
            if self.config.client_secret and path == "/oauth/token":
                client_str = self.config.client_id + ":" + self.config.client_secret
                b64Val = base64.b64encode(bytes(client_str, "utf-8"))
                headers["authorization"] = "Basic " + b64Val.decode("utf-8")
            elif self.config.client_secret:
                headers["authorization"] = "Bearer " + self.config.api_key
        # Add user-agent
        # version without v in user-agent
        # consistent with format of version in qlik-cli-user-agent
        version = __version__[1:]
        os_name = platform.system()
        headers["User-Agent"] = f"qlik-sdk-python/{version} ({os_name})"
        # If the data can be converted to a dict then send
        # it as json, otherwise send it as data.
        json_data = None
        if data and not isinstance(data, bytes):
            json_data = _get_dict(data)
            if json_data:
                data = None
        if params:
            params = _get_dict(params)
        # Create request.
        req = requests.Request(
            method,
            self.base_url.strip("/") + path,
            data=data,
            json=json_data,
            files=files,
            headers=headers,
            params=params,
        )

        req = reduce(lambda d, f: f(d), self._interceptors["request"].handlers, req)
        with requests.Session() as session:
            prepared = req.prepare()
            try:
                res = session.send(
                    prepared,
                    timeout=timeout,
                    stream=stream,
                )
            except requests.exceptions.Timeout:
                raise ConnectionException("Connection Timeout: " + self.base_url)
            except requests.exceptions.RequestException as exc:
                raise ConnectionException("Connection Error: " + self.base_url) from exc

            res = reduce(
                lambda r, f: f(r), self._interceptors["response"].handlers, res
            )

            try:
                res.raise_for_status()
            except Exception as e:
                res.close()
                if res.status_code == 401:
                    error = "Failed to authenticate"
                    try:
                        error = res.json()["errors"][0]["title"]
                    except Exception:
                        pass
                    raise AuthenticationException(error)
                raise e

            return res


class RestClientInstance:
    interceptors: Interceptors[ResponseInterceptor, RequestInterceptor]

    def __init__(self, restClient: RestClient) -> None:
        self._restClient = restClient
        self.interceptors = restClient._interceptors

    def __call__(self, **all: requests.Request) -> requests.Response:
        return self._restClient.rest(**all)



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

import io
import json
from dataclasses import asdict, dataclass
from enum import Enum

from ..auth import Auth, Config
from ..listable import ListableResource
from ..utils import get_mime_type


class GetConnectionsSortField(Enum):
    SpaceId = "spaceId"
    AscSpaceId = "+spaceId"
    DescSpaceId = "-spaceId"


class GetDataFileInfosSortField(Enum):
    Name = "name"
    AscName = "+name"
    DescName = "-name"
    Size = "size"
    AscSize = "+size"
    DescSize = "-size"
    ModifiedDate = "modifiedDate"
    AscModifiedDate = "+modifiedDate"
    DescModifiedDate = "-modifiedDate"


@dataclass
class DataFileUploadResponse:
    """

    Attributes
    ----------
    appId: str
      If this file is bound to the lifecycle of a specific app, this is the ID of this app.
    createdDate: str
      The date that the uploaded file was created.
    id: str
      The ID for the uploaded file.
    modifiedDate: str
      The date that the updated file was last modified.
    name: str
      The name of the uploaded file.
    ownerId: str
      The 'owner' of a file is the user who last uploaded the file's content.
    size: int
      The size of the uploaded file, in bytes.
    spaceId: str
      If the file was uploaded to a team space, this is the ID of that space.
    """

    appId: str = None
    createdDate: str = None
    id: str = None
    modifiedDate: str = None
    name: str = None
    ownerId: str = None
    size: int = None
    spaceId: str = None

    def __init__(self_, **kvargs):
        if "appId" in kvargs and kvargs["appId"] is not None:
            self_.appId = kvargs["appId"]
        if "createdDate" in kvargs and kvargs["createdDate"] is not None:
            self_.createdDate = kvargs["createdDate"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "modifiedDate" in kvargs and kvargs["modifiedDate"] is not None:
            self_.modifiedDate = kvargs["modifiedDate"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "ownerId" in kvargs and kvargs["ownerId"] is not None:
            self_.ownerId = kvargs["ownerId"]
        if "size" in kvargs and kvargs["size"] is not None:
            self_.size = kvargs["size"]
        if "spaceId" in kvargs and kvargs["spaceId"] is not None:
            self_.spaceId = kvargs["spaceId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def change_owner(self, data: ChangeDataFileOwnerRequest) -> None:
        """
        Change the owner of an existing data file.
        This is primarily an admin type of operation.  In general, the owner of a data file is implicitly set as
        part of a data file upload.  For data files that reside in a personal space, changing the owner has the
        effect of moving the data file to the new owner's personal space.

        Parameters
        ----------
        data: ChangeDataFileOwnerRequest
          The request.
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        self.auth.rest(
            path="/data-files/{id}/actions/change-owner".replace("{id}", self.id),
            method="POST",
            params={},
            data=data,
        )

    def change_space(self, data: ChangeDataFileSpaceRequest = None) -> None:
        """
        Change the space that an existing data file resides in.
        This is to allow for a separate admin type of operation that is more global in terms of access in cases
        where admin users may not explicitly have been granted full access to a given space within the declared
        space-level permissions.  If the space ID is set to null, then the datafile will end up residing in the
        personal space of the user who is the owner of the file.

        Parameters
        ----------
        data: ChangeDataFileSpaceRequest = None
          The request.
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        self.auth.rest(
            path="/data-files/{id}/actions/change-space".replace("{id}", self.id),
            method="POST",
            params={},
            data=data,
        )

    def delete(self) -> None:
        """
        Delete the specified data file.

        Parameters
        ----------
        """
        self.auth.rest(
            path="/data-files/{id}".replace("{id}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def set(
        self,
        File: io.BufferedReader = None,
        name: str = None,
        appId: str = None,
        sourceId: str = None,
        connectionId: str = None,
        tempContentFileId: str = None,
    ) -> DataFileUploadResponse:
        """
        Re-upload an existing data file.

        Parameters
        ----------
        File: str = None
          IFormFile form multipart/form-data
        name: str = None
          Name that will be given to the uploaded file.  If this name is different than the name used when the file
          was last POSTed or PUT, this will result in a rename of the file.  It should be noted that the '/' character
          in a data file name indicates a 'path' separator in a logical folder hierarchy for the name.  Names that
          contain '/'s should be used with the assumption that a logical 'folder hierarchy' is being defined for the
          full pathname of that file.  '/' is a significant character in the data file name, and may impact the
          behavior of future APIs that take this folder hierarchy into account.
        appId: str = None
          If this file should be bound to the lifecycle of a specific app, this is the ID of this app.
        sourceId: str = None
          If a SourceId is specified, this is the ID of the existing data file whose content should be copied into
          the specified data file.  That is, instead of the file content being specified in the Data element,
          it is effectively copied from an existing, previously uploaded file.
        connectionId: str = None
          If present, this is the DataFiles connection that the upload should occur in the context of.  If absent,
          the default is that the upload will occur in the context of the MyDataFiles connection.  If the DataFiles
          connection is different from the one specified when the file was last POSTed or PUT, this will result in
          a logical move of this file into the new space.
        tempContentFileId: str = None
          If a TempContentFileId is specified, this is the ID of a previously uploaded temporary content file whose
          content should be copied into the specified data file.  That is, instead of the file content being specified
          in the Data element, it is effectively copied from an existing, previously uploaded file.  The expectation
          is that this file was previously uploaded to the temporary content service, and the ID specified here is
          the one returned from the temp content upload request.
        """
        files_dict = {}
        files_dict["File"] = ("File", File, get_mime_type(File))
        Json_dict = {}
        if name is not None:
            Json_dict["name"] = name
        if appId is not None:
            Json_dict["appId"] = appId
        if sourceId is not None:
            Json_dict["sourceId"] = sourceId
        if connectionId is not None:
            Json_dict["connectionId"] = connectionId
        if tempContentFileId is not None:
            Json_dict["tempContentFileId"] = tempContentFileId
        files_dict["Json"] = (None, json.dumps(Json_dict))
        response = self.auth.rest(
            path="/data-files/{id}".replace("{id}", self.id),
            method="PUT",
            params={},
            data=None,
            files=files_dict,
        )
        self.__init__(**response.json())
        return self


@dataclass
class BatchChangeSpaceItem:
    """

    Attributes
    ----------
    id: str
      The ID of the data file whose space will be changed.
    spaceId: str
      The ID of the new space.  Passing in a null will result in the data file being moved to the user's
      personal space.
    """

    id: str = None
    spaceId: str = None

    def __init__(self_, **kvargs):
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "spaceId" in kvargs and kvargs["spaceId"] is not None:
            self_.spaceId = kvargs["spaceId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class BatchDeleteItem:
    """

    Attributes
    ----------
    id: str
      The ID of the data file to delete.
    """

    id: str = None

    def __init__(self_, **kvargs):
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ChangeDataFileOwnerRequest:
    """

    Attributes
    ----------
    ownerId: str
      The ID of the new owner.
    """

    ownerId: str = None

    def __init__(self_, **kvargs):
        if "ownerId" in kvargs and kvargs["ownerId"] is not None:
            self_.ownerId = kvargs["ownerId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ChangeDataFileSpaceRequest:
    """

    Attributes
    ----------
    spaceId: str
      The ID of the space.  If null, this data file will be moved to the user's personal space.
    """

    spaceId: str = None

    def __init__(self_, **kvargs):
        if "spaceId" in kvargs and kvargs["spaceId"] is not None:
            self_.spaceId = kvargs["spaceId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ConnectionsResponse:
    """

    Attributes
    ----------
    connectStatement: str
      The connect statement that will be passed to the connector when invoked.
    id: str
      The unique identifier of the connection.
    name: str
      The name of the connection.
    spaceId: str
      The team space that the given connection is associated with.  If null, the connection is not associated
      with any specific team space.
    type: str
      The type of the connection.
    """

    connectStatement: str = None
    id: str = None
    name: str = None
    spaceId: str = None
    type: str = None

    def __init__(self_, **kvargs):
        if "connectStatement" in kvargs and kvargs["connectStatement"] is not None:
            self_.connectStatement = kvargs["connectStatement"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "spaceId" in kvargs and kvargs["spaceId"] is not None:
            self_.spaceId = kvargs["spaceId"]
        if "type" in kvargs and kvargs["type"] is not None:
            self_.type = kvargs["type"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DataFileBatchChangeSpaceRequest:
    """
    Specifies the list of data file change space operations in a single batch.

    Attributes
    ----------
    change_space: list[BatchChangeSpaceItem]
      The list of data files to delete.
    """

    change_space: list[BatchChangeSpaceItem] = None

    def __init__(self_, **kvargs):
        if "change-space" in kvargs and kvargs["change-space"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == DataFileBatchChangeSpaceRequest.__annotations__["change_space"]
                for e in kvargs["change-space"]
            ):
                self_.change_space = kvargs["change-space"]
            else:
                self_.change_space = [
                    BatchChangeSpaceItem(**e) for e in kvargs["change-space"]
                ]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class DataFileBatchDeleteRequest:
    """
    Specifies the list of data files to be deleted in a single batch.

    Attributes
    ----------
    delete: list[BatchDeleteItem]
      The list of data files to delete.
    """

    delete: list[BatchDeleteItem] = None

    def __init__(self_, **kvargs):
        if "delete" in kvargs and kvargs["delete"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == DataFileBatchDeleteRequest.__annotations__["delete"]
                for e in kvargs["delete"]
            ):
                self_.delete = kvargs["delete"]
            else:
                self_.delete = [BatchDeleteItem(**e) for e in kvargs["delete"]]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class GetConnectionsResponse:
    """

    Attributes
    ----------
    data: list[ConnectionsResponse]
      Properties of the connections to the tenant spaces.
    links: LinksResponse
    """

    data: list[ConnectionsResponse] = None
    links: LinksResponse = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == GetConnectionsResponse.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [ConnectionsResponse(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if (
                type(kvargs["links"]).__name__
                == GetConnectionsResponse.__annotations__["links"]
            ):
                self_.links = kvargs["links"]
            else:
                self_.links = LinksResponse(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class GetDataFileInfosResponse:
    """

    Attributes
    ----------
    data: list[DataFileUploadResponse]
      Properties of the uploaded data files.
    links: LinksResponse
    """

    data: list[DataFileUploadResponse] = None
    links: LinksResponse = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == GetDataFileInfosResponse.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [DataFileUploadResponse(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if (
                type(kvargs["links"]).__name__
                == GetDataFileInfosResponse.__annotations__["links"]
            ):
                self_.links = kvargs["links"]
            else:
                self_.links = LinksResponse(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class LinkResponse:
    """

    Attributes
    ----------
    href: str
      The URL for the link.
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class LinksResponse:
    """

    Attributes
    ----------
    next: LinkResponse
    prev: LinkResponse
    self: LinkResponse
    """

    next: LinkResponse = None
    prev: LinkResponse = None
    self: LinkResponse = None

    def __init__(self_, **kvargs):
        if "next" in kvargs and kvargs["next"] is not None:
            if type(kvargs["next"]).__name__ == LinksResponse.__annotations__["next"]:
                self_.next = kvargs["next"]
            else:
                self_.next = LinkResponse(**kvargs["next"])
        if "prev" in kvargs and kvargs["prev"] is not None:
            if type(kvargs["prev"]).__name__ == LinksResponse.__annotations__["prev"]:
                self_.prev = kvargs["prev"]
            else:
                self_.prev = LinkResponse(**kvargs["prev"])
        if "self" in kvargs and kvargs["self"] is not None:
            if type(kvargs["self"]).__name__ == LinksResponse.__annotations__["self"]:
                self_.self = kvargs["self"]
            else:
                self_.self = LinkResponse(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class MultiStatusResponse:
    """

    Attributes
    ----------
    data: list[MultiStatusResponseItem]
      List of individual results for the items in the specified batch.
    """

    data: list[MultiStatusResponseItem] = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == MultiStatusResponse.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [MultiStatusResponseItem(**e) for e in kvargs["data"]]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class MultiStatusResponseItem:
    """

    Attributes
    ----------
    code: str
      The error code.
    detail: str
      A human-readable explanation specific to this occurrence of the problem.
    id: str
      The unique identifier of the file.
    status: int
      The HTTP status code.
    title: str
      Summary of the problem.
    """

    code: str = None
    detail: str = None
    id: str = None
    status: int = None
    title: str = None

    def __init__(self_, **kvargs):
        if "code" in kvargs and kvargs["code"] is not None:
            self_.code = kvargs["code"]
        if "detail" in kvargs and kvargs["detail"] is not None:
            self_.detail = kvargs["detail"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        if "title" in kvargs and kvargs["title"] is not None:
            self_.title = kvargs["title"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class QuotaResponse:
    """

    Attributes
    ----------
    allowedExtensions: list[str]
      The allowed file extensions on files that are uploaded.
    allowedInternalExtensions: list[str]
      The allowed file extensions for files that are only used internally by the system (and thus not typically
      shown to end users).
    maxFileSize: int
      Maximum allowable size of an uploaded file.
    maxLargeFileSize: int
      Maximum allowable size for a single uploaded large data file (in bytes).  This is a file that was indirectly
      uploaded using the temp content service chunked upload capability.
    maxSize: int
      The maximum aggregate size of all files uploaded by a given user.
    size: int
      The current aggregate size of all files uploaded by a given user.  If the current aggregate size is greater
      than the maximum aggregate size, this is a quota violation.
    """

    allowedExtensions: list[str] = None
    allowedInternalExtensions: list[str] = None
    maxFileSize: int = None
    maxLargeFileSize: int = None
    maxSize: int = None
    size: int = None

    def __init__(self_, **kvargs):
        if "allowedExtensions" in kvargs and kvargs["allowedExtensions"] is not None:
            self_.allowedExtensions = kvargs["allowedExtensions"]
        if (
            "allowedInternalExtensions" in kvargs
            and kvargs["allowedInternalExtensions"] is not None
        ):
            self_.allowedInternalExtensions = kvargs["allowedInternalExtensions"]
        if "maxFileSize" in kvargs and kvargs["maxFileSize"] is not None:
            self_.maxFileSize = kvargs["maxFileSize"]
        if "maxLargeFileSize" in kvargs and kvargs["maxLargeFileSize"] is not None:
            self_.maxLargeFileSize = kvargs["maxLargeFileSize"]
        if "maxSize" in kvargs and kvargs["maxSize"] is not None:
            self_.maxSize = kvargs["maxSize"]
        if "size" in kvargs and kvargs["size"] is not None:
            self_.size = kvargs["size"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class DataFiles:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def change_space(
        self, data: DataFileBatchChangeSpaceRequest
    ) -> MultiStatusResponse:
        """
        Change the spaces that a set of existing data files reside in a a single batch.
        This is to allow for a separate admin type of operation that is more global in terms of access in cases
        where admin users may not explicitly have been granted full access to a given space within the declared
        space-level permissions.  If the space ID is set to null, then the data file will end up residing in the
        personal space of the user who is the owner of the file.

        Parameters
        ----------
        data: DataFileBatchChangeSpaceRequest
          Specifies the list of data file change space operations in a single batch.
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/data-files/actions/change-space",
            method="POST",
            params={},
            data=data,
        )
        obj = MultiStatusResponse(**response.json())
        obj.auth = self.auth
        return obj

    def delete(self, data: DataFileBatchDeleteRequest) -> MultiStatusResponse:
        """
        Delete the specified set of data files as a single batch.

        Parameters
        ----------
        data: DataFileBatchDeleteRequest
          Specifies the list of data files to be deleted in a single batch.
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/data-files/actions/delete",
            method="POST",
            params={},
            data=data,
        )
        obj = MultiStatusResponse(**response.json())
        obj.auth = self.auth
        return obj

    def get_connection(self, id: str) -> ConnectionsResponse:
        """
        Get the built-in connection used by the engine to load/write data files given a connection ID.

        Parameters
        ----------
        id: str
          The ID of the connection.
        """
        response = self.auth.rest(
            path="/data-files/connections/{id}".replace("{id}", id),
            method="GET",
            params={},
            data=None,
        )
        obj = ConnectionsResponse(**response.json())
        obj.auth = self.auth
        return obj

    def get_connections(
        self,
        appId: str = None,
        limit: int = 20,
        name: str = None,
        page: str = None,
        personal: bool = False,
        sort: GetConnectionsSortField = None,
        spaceId: str = None,
    ) -> ListableResource[ConnectionsResponse]:
        """
        Get the list of built-in connections used by the engine to load/write data files.
        The non-filtered list contains a set of hardcoded connections, along with one connection per team space that
        the given user has access to.

        Parameters
        ----------
        appId: str = None
          If present, get connections with connection strings that are scoped to the given app ID.
        limit: int = 20
          If present, the maximum number of data file connection records to return.
        name: str = None
          If present, only return connections with the given name.
        page: str = None
          If present, the cursor that starts the page of data that is returned.
        personal: bool = False
          If true, only return the connections that access data in a personal space.  Default is false.
        sort: Literal["spaceId", "+spaceId", "-spaceId"] = None
          The name of the field used to sort the result.  By default, the sort is ascending.  Putting a '+' prefix on
          the sort field name explicitly indicates ascending sort order.  A '-' prefix indicates a descending sort order.
        spaceId: str = None
          If present, only return the connection that accesses data files in the specified space.
        """
        query_params = {}
        if appId is not None:
            query_params["appId"] = appId
        if limit is not None:
            query_params["limit"] = limit
        if name is not None:
            query_params["name"] = name
        if page is not None:
            query_params["page"] = page
        if personal is not None:
            query_params["personal"] = personal
        if sort is not None:
            query_params["sort"] = sort
        if spaceId is not None:
            query_params["spaceId"] = spaceId
        response = self.auth.rest(
            path="/data-files/connections",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=ConnectionsResponse,
            auth=self.auth,
            path="/data-files/connections",
            query_params=query_params,
        )

    def get_quotas(self) -> QuotaResponse:
        """
        Get quota information for the calling user.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/data-files/quotas",
            method="GET",
            params={},
            data=None,
        )
        obj = QuotaResponse(**response.json())
        obj.auth = self.auth
        return obj

    def get(self, id: str) -> DataFileUploadResponse:
        """
        Get descriptive info for the specified data file.

        Parameters
        ----------
        id: str
          The ID of the data file.
        """
        response = self.auth.rest(
            path="/data-files/{id}".replace("{id}", id),
            method="GET",
            params={},
            data=None,
        )
        obj = DataFileUploadResponse(**response.json())
        obj.auth = self.auth
        return obj

    def get_data_files(
        self,
        allowInternalFiles: bool = False,
        appId: str = None,
        connectionId: str = None,
        limit: int = 20,
        name: str = None,
        ownerId: str = None,
        page: str = None,
        sort: GetDataFileInfosSortField = None,
    ) -> ListableResource[DataFileUploadResponse]:
        """
        Get descriptive info for the specified data files.

        Parameters
        ----------
        allowInternalFiles: bool = False
          If set to false, do not return data files with internal extensions else return all the data files.
        appId: str = None
          Only return files scoped to the specified app.  If this parameter is not specified, only files that are not
          scoped to any app are returned.  "*" implies all app-scoped files are returned.
        connectionId: str = None
          Return files that reside in the space referenced by the specified DataFiles connection.  If this parameter
          is not specified, the user's personal space is implied.
        limit: int = 20
          If present, the maximum number of data files to return.
        name: str = None
          Filter the list of files returned to the given file name.
        ownerId: str = None
          If present, fetch the data files for the specified owner.  If a connectionId is specified in this case, the
          returned list is constrained to the specified space.  If connectionId is not specified, then all files owned
          by the specified user are returned regardless of the personal space that a given file resides in.
        page: str = None
          If present, the cursor that starts the page of data that is returned.
        sort: Literal["name", "+name", "-name", "size", "+size", "-size", "modifiedDate", "+modifiedDate", "-modifiedDate"] = None
          The name of the field used to sort the result.  By default, the sort order is ascending.  Putting a '+' prefix on
          the sort field name explicitly indicates ascending sort order.  A '-' prefix indicates a descending sort order.
        """
        query_params = {}
        if allowInternalFiles is not None:
            query_params["allowInternalFiles"] = allowInternalFiles
        if appId is not None:
            query_params["appId"] = appId
        if connectionId is not None:
            query_params["connectionId"] = connectionId
        if limit is not None:
            query_params["limit"] = limit
        if name is not None:
            query_params["name"] = name
        if ownerId is not None:
            query_params["ownerId"] = ownerId
        if page is not None:
            query_params["page"] = page
        if sort is not None:
            query_params["sort"] = sort
        response = self.auth.rest(
            path="/data-files",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=DataFileUploadResponse,
            auth=self.auth,
            path="/data-files",
            query_params=query_params,
        )

    def create(
        self,
        File: io.BufferedReader = None,
        name: str = None,
        appId: str = None,
        sourceId: str = None,
        connectionId: str = None,
        tempContentFileId: str = None,
    ) -> DataFileUploadResponse:
        """
        Upload a new data file.

        Parameters
        ----------
        File: str = None
          IFormFile form multipart/form-data
        name: str = None
          Name that will be given to the uploaded file.  It should be noted that the '/' character
          in a data file name indicates a 'path' separator in a logical folder hierarchy for the name.  Names that
          contain '/'s should be used with the assumption that a logical 'folder hierarchy' is being defined for the
          full pathname of that file.  '/' is a significant character in the data file name, and may impact the
          behavior of future APIs which take this folder hierarchy into account.
        appId: str = None
          If this file should be bound to the lifecycle of a specific app, this is the ID of this app.
        sourceId: str = None
          If a SourceId is specified, this is the ID of the existing data file whose content should be copied into
          the specified data file.  That is, instead of the file content being specified in the Data element,
          it is effectively copied from an existing, previously uploaded file.
        connectionId: str = None
          If present, this is the DataFiles connection that the upload should occur in the context of.  If absent,
          the default is that the upload will occur in the context of the MyDataFiles connection.  If the DataFiles
          connection is different from the one specified when the file was last POSTed or PUT, this will result in
          a logical move of this file into the new space.
        tempContentFileId: str = None
          If a TempContentFileId is specified, this is the ID of a previously uploaded temporary content file whose
          content should be copied into the specified data file.  That is, instead of the file content being specified
          in the Data element, it is effectively copied from an existing, previously uploaded file.  The expectation
          is that this file was previously uploaded to the temporary content service, and the ID specified here is
          the one returned from the temp content upload request.
        """
        files_dict = {}
        files_dict["File"] = ("File", File, get_mime_type(File))
        Json_dict = {}
        if name is not None:
            Json_dict["name"] = name
        if appId is not None:
            Json_dict["appId"] = appId
        if sourceId is not None:
            Json_dict["sourceId"] = sourceId
        if connectionId is not None:
            Json_dict["connectionId"] = connectionId
        if tempContentFileId is not None:
            Json_dict["tempContentFileId"] = tempContentFileId
        files_dict["Json"] = (None, json.dumps(Json_dict))
        response = self.auth.rest(
            path="/data-files", method="POST", params={}, data=None, files=files_dict
        )
        obj = DataFileUploadResponse(**response.json())
        obj.auth = self.auth
        return obj
